/*
 *	    ____  ____  _____   ____________
 *	   / __ / __ /  _/ | / /_  __/ __
 *	  / /_/ / /_/ // //  |/ / / / / /_/ /
 *	 / ____/ _, _// // /|  / / / / _, _/
 *	/_/   /_/ |_/___/_/ |_/ /_/ /_/ |_|
 *
 */

/*
 *	Publish an app to the Formide App Store
 */

var fs 				= require('fs');
var	path			= require('path');
var archiver 		= require('archiver');
var FormideProject 	= require('./project.js');
var Task 			= require('./task.js').Task;
var FormideLogin 	= require('./login.js').FormideTask;
var FormideUtils 	= require('./utils.js');
var request			= require('request');
var FormData 		= require('form-data');
var parseUrl 		= require('url').parse;

var FormideTask 	= function() {};

FormideTask.prototype = new Task();

FormideTask.prototype.run = function( formide, callback )
{
	var project = null;
	this.formide = formide;

	try
	{
		project = FormideProject.load();
	}
	catch( e )
	{
		this.formide.fail( e.message );
	}

	var self = this;

	var login = new FormideLogin();

	login.get( formide, function( jar )
	{
		var homeDir = process.env.HOME || process.env.USERPROFILE || process.env.HOMEPATH;
		var TEMP_FILENAME = path.join(homeDir, '.formide', project.get('identifier') + '.zip');
		var zip = fs.createWriteStream(TEMP_FILENAME);
		var archive = archiver('zip');

		archive.pipe(zip);

		archive.bulk([
			{
				expand: true,
				cwd: './',
				src: ['**']
			}
		]);

		archive.finalize( function( err, bytes )
		{
			if( err )
			{
				return formide.fail('Error uploading: ' + err);
			}
		});

		zip.on('close', function()
		{
			console.log('\nUploading app...'.bold.green);

			var csrftoken = '';
			csrftoken = FormideUtils.retrieveCsrfToken(jar);
			var url = formide.FORMIDE_API + formide.API_APPS + '/upload/' + project.get('app_id');

			var form = new FormData();
			form.append('csrfmiddlewaretoken', csrftoken);
			form.append('app_file', fs.createReadStream(TEMP_FILENAME, {filename: project.get('app_id'), contentType: 'application/zip'}));

			var params = parseUrl(url);

			form.submit({
				protocol: params.protocol,
				hostname: params.hostname,
				port: params.port,
				path: params.path,
				headers: form.getHeaders(
				{
					cookie: jar.map(function(c)
					{
						return c.key + "=" + encodeURIComponent(c.value);
          			}).join("; ")
        		})
			}, function( err, response )
			{
				if( err )
				{
					formide.fail('Uploading failed: ' + err)
				}

				var data = "";
				response.on('data', function( chunk )
				{
					data += chunk;
        		});

        		response.on('end', function()
        		{
	        		var response = JSON.parse(data);
	        		if(response.success)
	        		{
		        		console.log( response.message.bold.green );
	        		}
	        		else
	        		{
		        		console.log( response.message.bold.red );
	        		}
        		});
			});

/*
			var upload = request.post( url );
			var form =upload.form();
			form.append('file', fs.createReadStream( TEMP_FILENAME ));

			upload.on('end', function()
			{
				console.log(upload.response);
				if(upload.response.statusCode == 200)
				{
					console.log('\nFinished uploading'.bold.green);
					console.log( ('Go to https://apps.formide.com/apps/'+project.get('app_id')+' to test or publish your app').yellow );
				}
				else
				{
					formide.fail('Uploading failed: ' + upload.response.statusCode)
				}
			});
*/
		});
	});
};

exports.FormideTask = FormideTask;